@@ -27,6 +27,7 @@ using namespace std;
 
 static unsigned localport=0;
 static unsigned remoteport=0;
+static FILE *ipm, *ipc, *tcp;
 
 static string RespType;
 static string AddInfo;
@@ -114,7 +115,6 @@ static int Forward(DWORD host, int lport, int rport)
     }
     char Buf[256];
     sprintf(Buf, "%d,%d\r\n", rport, lport);
-    sprintf(Buf, "%d,%d\r\n", rport, lport);
     msyslog(ll_verbose, "Forwarding to host %s the following request: %s\n",
             inet_ntoa(addr), Buf);
     write(sock, Buf, strlen(Buf));
@@ -128,7 +128,7 @@ static int Resolve(void)
 {
     FILE *fp;
     char Buf[256];
-    if((fp = fopen("/proc/net/ip_masquerade", "rt")) != NULL)
+    if((fp = ipm) != NULL)
     {
         while(fgets(Buf, 255, fp))
         {
@@ -146,7 +146,7 @@ static int Resolve(void)
         fclose(fp);
     }
     /* 2.4.-kernel support by Wesley W. Terpstra <wesley@terpstra.ca> */
-    if((fp = fopen("/proc/net/ip_conntrack", "rt")) != NULL)
+    if((fp = ipc) != NULL)
     {
         while(fgets(Buf, 255, fp))
         {
@@ -171,7 +171,7 @@ static int Resolve(void)
         }
         fclose(fp);
     }
-    if((fp = fopen("/proc/net/tcp", "rt")) != NULL)
+    if((fp = tcp) != NULL)
     {
         while(fgets(Buf, 255, fp))
         {
@@ -206,6 +206,17 @@ int main(int argc, const char *const *argv)
     const char *const a0 = *argv;
     int help=0, error=0;
     
+    /* Support for modern 2.4 and 2.6 kernels (root-only proc): */
+    
+    /* use root priveledges to open protected files */
+    ipm = fopen("/proc/net/ip_masquerade", "rt");
+    ipc = fopen("/proc/net/ip_conntrack", "rt");
+    tcp = fopen("/proc/net/tcp", "rt");
+    
+    /* drop priveledges */
+    setregid(65534, 65534); /* nogroup */
+    setreuid(65534, 65534); /* nobody */
+    
     loglevel = ll_normal;
     while(--argc > 0)
     {
@@ -229,7 +240,7 @@ int main(int argc, const char *const *argv)
     {
         fprintf(stderr,
           "Bisqwit's identd version "VERSION"\n"
-          "Copyright (C) 1992,2000 Bisqwit (http://iki.fi/bisqwit/)\n"
+          "Copyright (C) 1992,2005 Bisqwit (http://iki.fi/bisqwit/)\n"
           "\n"
           "To be spawned from inetd. Usage:\n"
           "    %s <options>\n"
